package com.example.appgain_io;

import androidx.annotation.NonNull;

import io.flutter.embedding.engine.plugins.FlutterPlugin;

import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry.Registrar;

import android.content.Context;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.content.pm.PackageManager.*;
import android.os.Bundle;
import android.util.Base64;
import android.util.Log;
import android.widget.Toast;

import androidx.annotation.NonNull;

import com.google.android.gms.common.GoogleApiAvailability;
import com.google.gson.Gson;

import io.appgain.sdk.controller.Appgain;
import io.appgain.sdk.interfaces.AppgainDataCallback;
import io.appgain.sdk.model.BaseResponse;
import io.appgain.sdk.model.deferredlinking.DeferredDeepLinkingResponse;

import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.*;

/** AppgainIoPlugin */
public class AppgainIoPlugin implements FlutterPlugin, MethodCallHandler {
  /// The MethodChannel that will the communication between Flutter and native Android
  ///
  /// This local reference serves to register the plugin with the Flutter Engine and unregister it
  /// when the Flutter Engine is detached from the Activity
  private MethodChannel channel;
  private Context context;
  Gson gson = new Gson();
  @Override
  public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
    channel = new MethodChannel(flutterPluginBinding.getBinaryMessenger(), "com.Appgain.io/AppgainSDK");
    channel.setMethodCallHandler(this);
    context = flutterPluginBinding.getApplicationContext();
  }

  @Override
  public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
//
    if (call.method.equals("getPlatformVersion")) {
      result.success("Android " + android.os.Build.VERSION.RELEASE);
    }
    else if (call.method.equals("initSDK")) {
      final String appId = call.argument("appId");
      final String apiKey = call.argument("apiKey");
      Appgain.enableLog();
      try {
        Appgain.initialize(context, appId, apiKey, true,
                new AppgainDataCallback<Void>() {
                  @Override
                  public void onSuccess(Void data) {
                    Log.d("Appgain", "initSDK");
                    result.success("done successfully");
                  }

                  @Override
                  public void onFailure(BaseResponse failure) {
                    Log.d("Appgain error", failure.toString());
                    result.error("error","Initialize not working",gson.toJson(failure));
                  }
                });
      } catch (Exception e) {
        e.printStackTrace();
      }
    }
    else if (call.method.equals("getUserID")) {
      try {
        String userId = Appgain.getUserId();
        result.success(userId);
      } catch (Exception e) {
        e.printStackTrace();
      }
    }
    else if (call.method.equals("matchLink")) {
      Appgain.enableLog();
      try {
        Appgain.matchLink(new AppgainDataCallback<DeferredDeepLinkingResponse>() {
          @Override
          public void onSuccess(DeferredDeepLinkingResponse response) {
            Log.d("Appgain", "matchLink");
            HashMap <String,Object> responseData = new HashMap<>();
            if(response.getExtraData() != null) {
              responseData.put("extra_data", response.getExtraData().toString());
            }else{
              responseData.put("extra_data", response.getExtraData());
            }
            responseData.put("smart_link_primary", response.getDeferredDeepLink());
            responseData.put("smart_link_id", response.getSmartLinkId());
            responseData.put("smart_link_url", response.getSmartLinkUrl());
            result.success(responseData);
          }

          @Override
          public void onFailure(BaseResponse failure) {
            Log.d("Appgain error", failure.toString());
            result.error("error","Match link not working",gson.toJson(failure));
          }
        });
      } catch (Exception e) {
        e.printStackTrace();
      }
    }
    else if (call.method.equals("fireAutomator")) {
      Appgain.enableLog();
      final String triggerPoint = call.argument("triggerPoint");
      Map<String, String> emptyMap = Collections.emptyMap();
      try {
        Appgain.fireAutomator(triggerPoint, emptyMap, new AppgainDataCallback<Void>() {
          @Override
          public void onSuccess(Void data) {
            result.success("successful");
          }

          @Override
          public void onFailure(BaseResponse failure) {
            Log.d("Appgain error", failure.toString());
            result.error("error","fireAutomator not working",gson.toJson(failure));
          }
        });
      } catch (NullPointerException e) {
        e.printStackTrace();
      }
    }
    else if (call.method.equals("fireAutomatorWithPersonalization")) {
      Appgain.enableLog();
      final String triggerPoint = call.argument("triggerPoint");
      Map<String, String> personalizationData = call.argument("personalizationMap");
      try {
        Appgain.fireAutomator(triggerPoint, personalizationData, new AppgainDataCallback<Void>() {
          @Override
          public void onSuccess(Void data) {
            result.success("successful");
          }

          @Override
          public void onFailure(BaseResponse failure) {
            Log.d("Appgain error", failure.toString());
            result.error("error","fireAutomatorWithPersonalization not working",gson.toJson(failure));

          }
        });
      } catch (Exception e) {
        e.printStackTrace();
      }
    }
    else if (call.method.equals("addPurchase")) {
      final String productName = call.argument("productName");
      final String currency = call.argument("currency");
      final double amount = call.argument("amount");
      try {
        Appgain.logPurchase(productName, (float) amount, currency, new AppgainDataCallback<Void>() {
          @Override
          public void onSuccess(Void data) {
            result.success("successful");
          }


          @Override
          public void onFailure(BaseResponse failure) {
            Log.d("Appgain error", failure.toString());
            result.error("error","Add purchase not working",gson.toJson(failure));

          }
        });
      } catch (Exception e) {
        e.printStackTrace();
      }
    }
    else if (call.method.equals("addNotificationChannel")) {
      final String channel = call.argument("notificationType");
      final String data = call.argument("item");
      try {
        Appgain.createNotificationChannel(channel, data, new AppgainDataCallback<Void>() {
          @Override
          public void onSuccess(Void data) {
            result.success("successful");
          }

          @Override
          public void onFailure(BaseResponse failure) {
            Log.d("Appgain error", failure.toString());
            result.error("error","addNotificationChannel not working",gson.toJson(failure));
          }
        });

      } catch (Exception e) {
        e.printStackTrace();
      }
    }
    else if (call.method.equals("cancelAutomator")) {
      final String triggerPoint = call.argument("triggerPoint");
      Appgain.cancelAutomator(triggerPoint, new AppgainDataCallback<Void>() {
        @Override
        public void onSuccess(Void data) {
          result.success("successful");
        }

        @Override
        public void onFailure(BaseResponse failure) {
          Log.d("Appgain error", failure.toString());
          result.error("error","cancelAutomator not working",gson.toJson(failure));
        }
      });

    }
    else if (call.method.equals("setUserId")) {
      final String userId = call.argument("userId");
      try {
        Appgain.updateUserId(userId, new AppgainDataCallback<String>() {
          @Override
          public void onFailure(BaseResponse failure) {
            Log.d("Appgain error", failure.toString());
            result.error("error","cancelAutomator not working",gson.toJson(failure));
          }
          @Override
          public void onSuccess(String newUserId) {
            Log.d("Appgain" ,newUserId);
            result.success(newUserId);
          }
        });
      } catch (Exception e) {
        e.printStackTrace();
      }
    }
    else if (call.method.equals("updateUser")) {
      Map<String, String> userData = call.argument("userData");
      try {
        Appgain.updateUserData(userData, new AppgainDataCallback<Void>() {
          @Override
          public void onSuccess(Void data) {
            result.success("successful");
          }

          @Override
          public void onFailure(BaseResponse failure) {
            Log.d("Appgain error", failure.toString());
            result.error("error","updateUser not working",gson.toJson(failure));
          }
        });
      } catch (Exception e) {
        e.printStackTrace();
      }
    }
    else if(call.method.equals("enableReciveNotification")){
      final boolean enable = call.argument("enable");
      final String type = call.argument("type");
      Appgain.enableNotifications(enable, type, new AppgainDataCallback<Void>() {
        @Override
        public void onSuccess(Void data) {
          result.success("successful");
        }

        @Override
        public void onFailure(BaseResponse failure) {
          Log.d("Appgain error", failure.toString());
          result.error("error","enableReciveNotification not working",gson.toJson(failure));

        }
      });
    }
    else if (call.method.equals("logEvent")) {
      final String action = call.argument("action");
      final String type = call.argument("type");
      Map<String, String> extras = new HashMap<String, String>();
      extras = call.argument("extras");
      Bundle bundle = new Bundle();

      for (String entry : extras.keySet()) {
        bundle.putString(entry, extras.get(entry));
      }
      try {
        Appgain.logEvent(type, action, bundle, new AppgainDataCallback<Void>() {
          @Override
          public void onSuccess(Void data) {
            result.success("successful");
          }

          @Override
          public void onFailure(BaseResponse failure) {
            Log.d("Appgain error", failure.toString());
            result.error("error","logEvent not working",gson.toJson(failure));
          }
        });
      } catch (Exception e) {
        e.printStackTrace();
      }
    } else {
      result.notImplemented();
    }
  }

  @Override
  public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
    channel.setMethodCallHandler(null);
  }
}
