import Flutter
import UIKit
import Appgain
public class SwiftAppgainIoPlugin: NSObject, FlutterPlugin {
    
   public static var platformAppgainSDK = "com.Appgain.io/AppgainSDK"
    var eventAppgainSDK : String{
        get{
            return "com.Appgain.io/AppgainSDKEvent"
        }
    }

    enum AppgainFunction : String {
        case initSDK
        case getUserID
        case matchLink
        case fireAutomatorWithPersonalization
        case fireAutomator
        case setUserId
        case addPurchase
        case enableReciveNotification
        case addNotificationChannel
        case updateUser
        case logEvent
        case cancelAutomator
        
        

 
       // final _updateChannel = 'updateChannel';
      

        
    }
    
  public static func register(with registrar: FlutterPluginRegistrar) {
    let channel = FlutterMethodChannel(name: platformAppgainSDK, binaryMessenger: registrar.messenger())
      
    let instance = SwiftAppgainIoPlugin()
    registrar.addMethodCallDelegate(instance, channel: channel)
  }

  public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
      
  
          switch call.method {
          case AppgainFunction.initSDK.rawValue:
              if   let data = call.arguments as? Dictionary<String, Any>{
                  
                  // var data =  {'appId':_textFieldController.value.text,'apiKey':_textFieldController2.value.text,'configure':true};
                  
                  Appgain.initialize(data["appId"] as? String, apiKey: data["apiKey"] as? String, trackUserForAdvertising: data["configure"] as! Bool) { (response, info, error) in
                      if info != nil {
                          result(info)
                      }
                      else{
                          result(error.debugDescription)
                      }
                  }
              }
              break
          case AppgainFunction.getUserID.rawValue:
              result(Appgain.getUserID())
              break
          case AppgainFunction.matchLink.rawValue:
              Appgain.matchLink { (response, info, error) in
                  if info != nil {
                      result(info)
                  }
                  else{
                      result(error.debugDescription)
                      
                  }
              }
              break
   
          case AppgainFunction.fireAutomatorWithPersonalization.rawValue:
              let data = call.arguments as? Dictionary<String, Any>
              Appgain.fireAutomator(data!["triggerPoint"] as? String, personalizationData: data!["personalizationMap"] as? NSMutableDictionary) { (response, info, error) in
                  if info != nil {
                      result(info)
                  }
                  else{
                      result(error.debugDescription)
                      
                  }
              }
              break
          case AppgainFunction.fireAutomator.rawValue:
              let data = call.arguments as? Dictionary<String, Any>
              Appgain.fireAutomator(data!["triggerPoint"] as? String, personalizationData: nil) { (response, info, error) in
                  if info != nil {
                      result(info)
                  }
                  else{
                      result(error.debugDescription)
                      
                  }
              }
              break
          case AppgainFunction.cancelAutomator.rawValue:
              let data = call.arguments as? Dictionary<String, Any>
              Appgain.cancelAutomator((data!["triggerPoint"] as? String) ?? "")  { (response, info, error) in
                  if info != nil {
                      result(info)
                  }
                  else{
                      result(error.debugDescription)
                      
                  }
              }
              break
          case AppgainFunction.setUserId.rawValue:
              if let data = call.arguments as? Dictionary<String, Any>{
                  
                  Appgain.updateUserId((data["userId"] as! String))
              }
              break
          case AppgainFunction.addPurchase.rawValue:
              if  let data = call.arguments as? Dictionary<String, Any>{
//data["amount"] as? Double  ?? 0
                  Appgain.logPurchase((data["productName"] as? String) ?? "" , withAmount: (data["amount"] as? Double )  ?? 0 , forCurrency: (data["currency"] as? String) ?? "" ) { (response, info, error) in
                      if info != nil {
                          result(info)
                      }
                      else{
                          result(error.debugDescription)
                      }
                  }
              }
              break
          case AppgainFunction.enableReciveNotification.rawValue:
              if  let data = call.arguments as? Dictionary<String, Any>{
                  
                  Appgain.enableNotifications((data["enable"] as! Bool), forType: (data["type"] as! String))
              }
              break
          case AppgainFunction.addNotificationChannel.rawValue:
              if  let data = call.arguments as? Dictionary<String, Any>{
                  
                  
                  // Appgain.createNotificationChannel((data["notificationType"] as! String), withData: (data["item"] as! String))
                  Appgain.createNotificationChannel((data["notificationType"] as! String), withData: (data["item"] as! String)) { (response, info, error) in
                      if info != nil {
                          result(info)
                      }
                      else{
                          result(error.debugDescription)
                          
                      }
                  }
              }
              break
          case AppgainFunction.updateUser.rawValue:
              if  let data = call.arguments as? Dictionary<String, Any>{
                  Appgain.updateUserData(data) { (response, info, error) in
                      if info != nil {
                          result(info)
                      }
                      else{
                          result(error.debugDescription)
                          
                      }
                  }
              }
              break
          case AppgainFunction.logEvent.rawValue:
              if  let data = call.arguments as? Dictionary<String, Any>{
                  Appgain.logEvent((data["type"] as? String) ?? "", andAction: ((data["action"] as? String) ) ?? "", extras: (data["extras"]   as? [AnyHashable : Any]) ?? [:]) { (response, info, error) in
                      if info != nil {
                          result(info)
                      }
                      else{
                          result(error.debugDescription)
                          
                      }
                  }
              }
              
              break
          default:
              break
          }
      }
}
